<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Setforms2\Shared\ORM\DateTime;

class ProdutoPreferencialController extends \Teknisa\GER\Controllers\TeknisaCrud {
    
    protected $connection;
    
    
    
     public function __construct($crudService, $parameterBag, $supUtilities, $connection)
    {
        parent::__construct($crudService, $parameterBag);
        $this->crudService  = $crudService;
        $this->supUtilities = $supUtilities;
        $this->connection = $connection;
        
        
    }
    
    public function concatArray($array){
        return is_array($array) ? join('_', $array) : 'T';
    }
    private function strPadRigthProd($value){
	    return str_pad($value, 20, 'Z', STR_PAD_RIGHT);
	}
    
    public function cadastramentoAutomatico(Request\Row $request, Response $response){
        $data = $request->getRow();
        
        $queryAutomaticExclusion = "DELETE FROM PRODPREFMED
                                        WHERE INSTR(:P_CDFILIDEST, PRODPREFMED.CDFILIAL) > 0
                                          AND EXISTS(SELECT 1
                                                        FROM PRODUTO
                                                      WHERE PRODUTO.CDPRODUTO = PRODPREFMED.CDPRODUTO
                                                        AND PRODUTO.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIM)";
        try {
            $param = [
                
                'P_CDFILIDEST' => self::concatArray($data['NMFILIALDEST']),
                'P_PRODINI'   => $data['CDPRODINI'],
                'P_PRODFIM' => self::strPadRigthProd($data['CDPRODFIN'])
                
            ];
            return $this->supUtilities->crud($queryAutomaticExclusion, $param);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw new \Exception($e->getMessage());
        }
    }
    
    public function exportarProdutos(Request\Row $request, Response $response){
        $row = $request->getRow();
        if($row['OPCOESEXPORTACAO'] === "DELETE_PRODUCTS") {
            $queryDeleteProducts = "DELETE FROM PRODPREFMED
                                  WHERE INSTR(:P_CDFILIDEST, PRODPREFMED.CDFILIAL) > 0
                                    AND EXISTS(SELECT 1
                                                  FROM PRODUTO
                                                WHERE PRODUTO.CDPRODUTO = PRODPREFMED.CDPRODUTO
                                                  AND PRODUTO.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIM)";
            
                $params = [
              
                'P_CDFILIDEST' => self::concatArray($row['NMFILIALDEST']),
                'P_PRODINI'   => $row['CDPRODINI'],
                'P_PRODFIM' => self::strPadRigthProd($row['CDPRODFIN'])
                
                ];
              
                $this->supUtilities->crud($queryDeleteProducts, $params);
                
            $queryInsertProducts = "INSERT INTO PRODPREFMED (CDFILIAL, CDPRODUTO, CDPRODPREF, QTMEDSOLPROD)
                                  SELECT FD.CDFILIAL, PR.CDPRODUTO, PP.CDPRODPREF, PP.QTMEDSOLPROD
                                    FROM FILIAL FD, PRODPREFMED PP
                                    JOIN PRODUTO PR ON PP.CDPRODUTO = PR.CDPRODUTO
                                  WHERE PP.CDFILIAL = :P_CDFILIORIG 
                                    AND PR.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIM 
                                    AND INSTR(:P_CDFILIDEST, FD.CDFILIAL) > 0";   
                                    
                $parametros = [
                'P_CDFILIORIG'   => $row['CDFILIALORIG'],
                'P_CDFILIDEST' => self::concatArray($row['NMFILIALDEST']),
                'P_PRODINI'   => $row['CDPRODINI'],
                'P_PRODFIM' => self::strPadRigthProd($row['CDPRODFIN'])
                
                ];                    
                
             $this->supUtilities->crud($queryInsertProducts, $parametros);
        } else {
            $queryInsertSql = "INSERT INTO PRODPREFMED (CDFILIAL, CDPRODUTO, CDPRODPREF, QTMEDSOLPROD)
                              SELECT FD.CDFILIAL, PR.CDPRODUTO, PP.CDPRODPREF, PP.QTMEDSOLPROD
                                FROM FILIAL FD, PRODPREFMED PP
                                JOIN PRODUTO PR ON PP.CDPRODUTO = PR.CDPRODUTO
                              WHERE PP.CDFILIAL = :P_CDFILIORIG 
                                AND PR.CDARVPROD BETWEEN :P_PRODINI AND :P_PRODFIM 
                                AND INSTR(:P_CDFILIDEST, FD.CDFILIAL) > 0
                                AND NOT EXISTS (SELECT 1
                                                  FROM PRODPREFMED
                                                  WHERE PRODPREFMED.CDFILIAL = FD.CDFILIAL
                                                    AND PRODPREFMED.CDPRODUTO = PP.CDPRODUTO)";
                                                    
            $parametro = [
                'P_CDFILIORIG'   => $row['CDFILIALORIG'],
                'P_CDFILIDEST' => self::concatArray($row['NMFILIALDEST']),
                'P_PRODINI'   => $row['CDPRODINI'],
                'P_PRODFIM' => self::strPadRigthProd($row['CDPRODFIN'])
                
                ];
                                                    
            return $this->supUtilities->crud($queryInsertSql, $parametro);
                                                    
        }
    }
}