<?php

namespace Modules\Fornecimento\Modules\IntegracaoAF\Controllers;

use Modules\Fornecimento\Modules\IntegracaoAF\Services\IntegracaoAFService;
use Controllers\ApprovalWorkflowController;

class IntegracaoAFController {

    protected $service;
    protected $approvalWorkflowController;
    
    protected $listaErrosPedido = array();
    protected $listaItensValidos = array();
    
    const CDFILLANC = '9999';
    const CDOPERADOR = '000000000987';
    const CDMARCA = '00680';
    const TIPOLANCAAF = 'A';

    public function __construct(IntegracaoAFService $service, ApprovalWorkflowController $approvalWorkflowController) {
        
        $this->service = $service;
        $this->approvalWorkflowController = $approvalWorkflowController;
        
	}
	
	function retornaArrayPost() {
        
        $json = file_get_contents('php://input');
	    $array = json_decode($json, true);
	    
	    if(!$array) {
	        throw new \Exception("Json inválido.", 1);
	    }
	    
	    return $array;
	    
	}
	
	function verificaIndicesArray($listaPedidos) {
		
		$indicesFornecedor = array('cnpj', 'nome', 'chaveAcesso');
		$indicesPedidos = array('numero', 'data', 'cnpjFornecedor', 'unidadeDestino', 'cnpjEmpresa');
		$indicesItens = array('codCliente', 'quantidade', 'vlrUnitario');
		$indicesFaltando = array();
		
		if(array_key_exists('fornecedor', $listaPedidos)) {
			self::chaveExiste($indicesFaltando, $indicesFornecedor, $listaPedidos['fornecedor'], 'fornecedor');
		} else {
			array_push($indicesFaltando, 'fornecedor');
		}
		
		if(array_key_exists('pedidos', $listaPedidos)) {
			$contaPedido = 1;
			foreach($listaPedidos['pedidos'] as $pedido) {
				
				self::chaveExiste($indicesFaltando, $indicesPedidos, $pedido, "pedidos{$contaPedido}");

				if(array_key_exists('itens', $pedido)) {
					$contaItem = 1;
					foreach($pedido['itens'] as $item) {
						
						self::chaveExiste($indicesFaltando, $indicesItens, $item, "pedidos{$contaPedido}->itens{$contaItem}");
						$contaItem++;
						
					}
				} else {
					array_push($indicesFaltando, "pedidos{$contaPedido}->itens");
				}
				
				$contaPedido++;
			}
		} else {
			array_push($indicesFaltando, "pedidos");
		}
		
		if (is_array($indicesFaltando) && count($indicesFaltando)) {
			$indicesFaltando = json_encode($indicesFaltando, JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE);
			throw new \Exception("Os seguinte rótulos não existem no arquivo enviado: {$indicesFaltando}", 1);
		}
		
	}
	
	function chaveExiste(&$indicesFaltando, $arrayIndice, $array, $caminho) {
		
		foreach($arrayIndice as $indice) {
			if(!array_key_exists($indice, $array)) {
				
				array_push($indicesFaltando, $caminho . '->' .$indice);
				
			}
		}
		
	}
	
	public function integracaoAFSup() {
	
	    try {
	        
	        $listaPedidos = self::retornaArrayPost();
	        self::verificaIndicesArray($listaPedidos);
	        
	        $errosPedidos = array();
	        $linkRelatorioErros = array();
	        
	        $fornecedor = $listaPedidos['fornecedor'];
	        $pedidos = $listaPedidos['pedidos'];
	        
	        $acessoFornecedor = self::validaAcessoFornecedor($fornecedor['cnpj'], $fornecedor['chaveAcesso']);
	        
	        foreach($pedidos as $pedido) {
	        	
	        	$this->service->iniciaTransacao();
	        	
	        	self::validaPedido($pedido['cnpjFornecedor'], $pedido['numero'], '9999', $pedido['unidadeDestino'], $pedido['data'], $pedido['cnpjEmpresa'], $pedido['itens']);
	        	self::processaPedidoValido($pedido);
	        	self::processaErrosPedido($errosPedidos, $linkRelatorioErros, $pedido);
	        	
	        	$this->service->commit();
	        	
	        }
	        
	        self::enviaEmailRelatorioErros($linkRelatorioErros, $acessoFornecedor, $fornecedor);
	        
	        var_dump(json_encode(array("pedidos" => $errosPedidos), JSON_PRETTY_PRINT | JSON_UNESCAPED_UNICODE));
	        
	    } catch(\Exception $e) {
			switch($e->getCode()) {
				case 1:
					var_dump($e->getMessage());
					break;
				default:
					$this->service->rollback();
					throw new \Exception($e->getMessage());
					break;
			}
	    }
	    
	}
	
	function validaAcessoFornecedor($cnpj, $chaveAcesso) {
	    
	    $acesso = $this->service->retornaUsrParamIntegra($cnpj);
	    
	    // if(count($acesso)) {
	    if (is_array($acesso) && count($acesso)) {	
	        if($acesso[0]['DSCHAVEACESSO'] !== $chaveAcesso) {
	        	throw new \Exception("Chave de acesso inválida.", 1);
	        }
	        
	    } else {
	    	throw new \Exception("Fornecedor não existe ou não está autorizado para este tipo de transação.", 1);
	    }
	    
	    return $acesso;
	    
	}
	
	function validaPedido($cnpjFornecedor, $numeroPedido, $unidadeLancamento, $unidadeDestino, $dataPedido, $cnpjEmpresa, $itens) {
		
		$this->listaErrosPedido = array();
    	$this->listaItensValidos = array();
		
		try {
		
			self::verificaPedidoExiste($cnpjFornecedor, $numeroPedido, $unidadeLancamento, $unidadeDestino);
			self::verificaFornecedorExiste($cnpjFornecedor);
			self::verificaUnidadeDestExiste($unidadeDestino);
			self::verificaEmpresaExiste($cnpjEmpresa);
			self::validaItensPedido($numeroPedido, $cnpjFornecedor, $unidadeDestino, $dataPedido, $itens);
			
			return true;
			
		} catch(\Exception $e) {
			switch($e->getCode()) {
				case 1:
					$this->listaErrosPedido = array( "numero" => $numeroPedido, "erro" => $e->getMessage() );
					break;
				default:
					throw new \Exception($e->getMessage());
					break;
			}
		}
		
	}
	
	function verificaPedidoExiste($cnpjFornecedor, $numeroPedido, $unidadeLancamento, $unidadeDestino) {
		
		$af = $this->service->retornaAutofornImportacao($cnpjFornecedor, $numeroPedido, $unidadeLancamento, $unidadeDestino);

		// if(count($af)) {
		if (is_array($af) && count($af)) {
			throw new \Exception("AF já foi incluída no sistema.", 1);
		}
		
		return true;
	}
	
	function verificaFornecedorExiste($cnpjFornecedor) {
		
		$fornecedor = $this->service->retornaFornecedor($cnpjFornecedor);
		
		// if(!count($fornecedor)) {
		if (!is_array($fornecedor) && !count($fornecedor)) {
			throw new \Exception("O fornecedor não existe.", 1);
		}
		
		return true;
	}
	
	function verificaEmpresaExiste($cnpjEmpresa) {
		
		$empresa = $this->service->retornaEmpresa($cnpjEmpresa);
		
		// if(!count($empresa)) {
		if (!is_array($empresa) && !count($empresa)) {
			throw new \Exception("Não existe empresa para o cadastro {$cnpjEmpresa}.", 1);
		}
		
		return true;
	}
	
	function verificaUnidadeDestExiste($unidadeDestino) {
		
		$unidadeDest = $this->service->retornaUnidade($unidadeDestino);
		
		// if(!count($unidadeDest)) {
		if (!is_array($unidadeDest) && !count($unidadeDest)) {
			throw new \Exception("Unidade de destino não existe.", 1);
		}
		
		return true;
	}
	
	function validaItensPedido($numeroPedido, $cnpjFornecedor, $unidadeDestino, $dataPedido, $itens) {
		
		$listaErrosItens = [];
		
		foreach($itens as $item) {
			
			$errosItem = [];
			
			$existeProduto = self::verificaProdutoExiste($item['codCliente'], $errosItem);
			
			if($existeProduto) {
			
				self::verificaProdutoUnico($itens, $item['codCliente'], $errosItem);
				self::verificaValorPositivo($item['quantidade'], $errosItem, 'Quantidade');
				self::verificaValorPositivo($item['vlrUnitario'], $errosItem, 'Valor Unitário');
				self::verificaProdutoFornecedor($cnpjFornecedor, $item['codCliente'], $errosItem);
				self::verificaMelhorPreco($cnpjFornecedor, $unidadeDestino, $item['codCliente'], $dataPedido, $item['vlrUnitario'], $errosItem);
			}
			
			// if(count($errosItem)) {
			if (is_array($errosItem) && count($errosItem)) {
				array_push($listaErrosItens, array( "codCliente" => $item['codCliente'], "erros" => $errosItem));
			} else {
				array_push($this->listaItensValidos, $item);
			}
		}
		
		// if(count($listaErrosItens)) {
		if (is_array($listaErrosItens) && count($listaErrosItens)) {			
			$this->listaErrosPedido = array( "numero" => $numeroPedido, "itens" => $listaErrosItens );
			return false;
		}
		
		return true;
	}
	
	function verificaProdutoExiste($codCliente, &$errosItem) {
		
		$produto = $this->service->retornaProduto($codCliente);
		
		// if(!count($produto)) {
		if (!is_array($produto) && !count($produto)) {			
			
			array_push($errosItem, "{$codCliente} - Código do produto não existe");
			
			return false;
		}
		
		return true;
	}
	
	function verificaProdutoUnico($itens, $codCliente, &$errosItem) {
		
		$existe = 0;
		
		foreach($itens as $item) {
			
			if($item['codCliente'] === $codCliente) {
				$existe++;
			}
			
		}
		
		if($existe > 1) {
			
			array_push($errosItem, "Produto repetido na nota");
			
			return false;
		} 
		
		return true;
		
	}
	
	function verificaValorPositivo($valor, &$errosItem, $nome) {
		
		if($valor <= 0) {
			
			array_push($errosItem, "{$nome} menor ou igual a zero");
			
			return false;
			
		}
		
		return true;
		
	}
	
	function verificaProdutoFornecedor($cnpjFornecedor, $codCliente, &$errosItem) {
		
		$prodForn = $this->service->retornaProdutoFornecedor($cnpjFornecedor, $codCliente);
		
		if(!$prodForn[0]['CDPRODCOMPL']) {
			
			array_push($errosItem, "Produto não cadastrado para o fornecedor");
			
			return false;
			
		}
		
		return true;
		
	}
	
	function verificaMelhorPreco($cnpjFornecedor, $unidadeDestino, $codCliente, $dataPedido, $vlrUnitario, &$errosItem) {
		
		$centralCmp = $this->service->retornaCentralCompras($unidadeDestino);
		$melhorPreco = $this->service->retornaMelhorPreco($centralCmp, $cnpjFornecedor, $unidadeDestino, $codCliente, $dataPedido);
		
		if($melhorPreco) {
		
			if($melhorPreco == $vlrUnitario) {
				return true;
			} else {
				
				array_push($errosItem, "Preço na nota diferente do melhor preço do sistema");
				
				return false;
			}
			
		} else {
			
			array_push($errosItem, "Não existe melhor preço cadastrado");
			
			return false;
		}
		
	}
	
	function processaPedidoValido($pedido) {
		
		if(count($this->listaItensValidos)) {
    		$nrautoforn = self::inserePedido($pedido['cnpjFornecedor'], $pedido['cnpjEmpresa'], $pedido['unidadeDestino'], $pedido['numero'], $pedido['data'], $this->listaItensValidos);
    		$this->service->executeProcedureFechamentoAF($pedido['unidadeDestino'], self::CDFILLANC, $nrautoforn);
			self::insereAFWorkflow($pedido['unidadeDestino'], $nrautoforn);
			
    	}
	}
	
	function processaErrosPedido(&$errosPedidos, &$linkRelatorioErros, $pedido){
		
		if(count($this->listaErrosPedido)) {
	        		
    		array_push($errosPedidos, $this->listaErrosPedido);
    		self::insereLog($pedido['numero'], $pedido['unidadeDestino'], $pedido['data']);
    		$linkRelatorioErros[$pedido['numero']] = self::geraRelatorioErros($pedido['numero'], $pedido['unidadeDestino']);
    		
    	}
		
	}
	
	function inserePedido($cnpjFornecedor, $cnpjEmpresa, $unidadeDestino, $numeroPedido, $dataPedido, $itens) {
		
		$nrautoforn = $this->service->geraNovoCodigoSC('AUTOFORNE');
		$fornecedor = $this->service->retornaFornecedor($cnpjFornecedor)[0];
		
		self::insereAF($nrautoforn, $fornecedor, $cnpjEmpresa, $unidadeDestino, $numeroPedido);
		self::insereProdutos($unidadeDestino, $nrautoforn, $dataPedido, $itens);
		self::inserePrazopgtaf($nrautoforn, $fornecedor, $unidadeDestino);
		self::insereMensagem($nrautoforn, $unidadeDestino, $numeroPedido);
		
		return $nrautoforn;
		
	}
	
	function insereAF($nrautoforn, $fornecedor, $cnpjEmpresa, $unidadeDestino, $numeroPedido) {
		
		$cdempresa = $this->service->retornaEmpresa($cnpjEmpresa)[0]['CDEMPRESA'];
		
		$this->service->insereAutoforne(self::CDFILLANC, self::CDOPERADOR, self::TIPOLANCAAF, $cdempresa, $unidadeDestino, $nrautoforn, $fornecedor, $numeroPedido);
		
	}
	
	function insereProdutos($unidadeDestino, $nrautoforn, $dataPedido, $itens) {
		
		foreach($itens as $item) {
			$this->service->insereItemautaf(self::CDFILLANC, $unidadeDestino, $nrautoforn, $dataPedido, self::CDMARCA, $item);
		}
		
	}
	
	function inserePrazopgtaf($nrautoforn, $fornecedor, $unidadeDestino) {
		
		if($fornecedor['IDTPFORPGTFO'] == 'V') {
			$this->service->inserePrazopgtaf($unidadeDestino, self::CDFILLANC, $nrautoforn, $fornecedor['CDFORNECED']);
		}
		
	}
	
	function insereMensagem($nrautoforn, $unidadeDestino, $numeroPedido) {
		
		$nrmensautfor = '01';
		$dsmensaf = "Pedido {$numeroPedido} realizado via importação de AF.";
		
		$this->service->insertMensagem($nrmensautfor, self::CDFILLANC, $nrautoforn, $dsmensaf, $unidadeDestino);
		
	}
	
	function insereAFWorkflow($unidadeDestino, $nrAutoForn) {
		
		$centralCmp = $this->service->retornaCentralCompras($unidadeDestino);
		$utilizaWorkflow = $this->approvalWorkflowController->utilizaWorkflow($centralCmp);

		if($utilizaWorkflow == 'S') {
			
            $operacao = 'LANCAMENTO';
            $tipoPedido = 'AF'; 
            
            $this->approvalWorkflowController->insertOrderOnWorkflow(self::CDFILLANC, $unidadeDestino, $nrAutoForn, $operacao, $tipoPedido);
			
		}
		
	}
	
	function insereLog($numeroPedido, $unidadeDestino, $dataPedido) {
		
		$this->service->deleteLog($numeroPedido, $unidadeDestino, $dataPedido);
		
		if(isset($this->listaErrosPedido['erro'])) {
			
			$this->service->insertLog($numeroPedido, $unidadeDestino, $dataPedido, '', $this->listaErrosPedido['erro']);
			
		} else if(isset($this->listaErrosPedido['itens'])) {
			
			foreach($this->listaErrosPedido['itens'] as $item){
				
				foreach($item['erros'] as $erro) {
					
					$codCliente = strstr($erro, 'produto não existe') ? '' : $item['codCliente'];
					
					$this->service->insertLog($numeroPedido, $unidadeDestino, $dataPedido, $codCliente, $erro);
					
				}
				
			}
			
		}
		
	}
	
	function geraRelatorioErros($numeroPedido, $unidadeDestino) {
		
		$relatorio = $this->service->geraRelatorioErros($numeroPedido, $unidadeDestino);
		
		return $relatorio;
		
	}
	
	function enviaEmailRelatorioErros($linkRelatorioErros, $acessoFornecedor, $fornecedor) {

		// if(count($linkRelatorioErros)) {
		if (is_array($linkRelatorioErros) && count($linkRelatorioErros)) {

			$operador = $this->service->retornaOperador($acessoFornecedor[0]['CDOPERADOR']);
			$destinatario = count($operador) ? $operador[0]['DSEMAILOPER'] : false;
			
			$assunto = "Erro de Importação";
			$mensagem = "";
			$fornecedor = $fornecedor['cnpj'] . ' - ' . $fornecedor['nome'];
			
			foreach($linkRelatorioErros as $numero=>$link) {
				$mensagem .= "<a href='" . $link . "' target='_blank'>" . $numero . "</a></br>";
			}
			
			$email = "<!DOCTYPE html><html><head> <meta charset='UTF-8'> <style>body{margin: 0px; display: block; padding: 0px; margin: 0px; font-family: Arial, sans-serif;}.containerGeral{background-color: #FFFFFF;}.linha{width: 590px; display: table;}.cabecalho{background-color: #FFFFFF; padding: 5px 0px 5px 0px;}.tipoEmail{background-color: #e6f2ff;}.conteudo{display: table-row; width: 550px; background-color: #FFFFFF;}.rodape{background-color: #2D68C4;}.logomarca{padding: 15px; vertical-align: middle;}#assunto{padding: 25px 50px 25px 50px; vertical-align: middle; text-align: left; color: #00305A; font-size: 23px;}#borda{vertical-align: middle;}#quadroConteudo{width: 550px; line-height: 150%; font-size: 14px; padding: 20px; color: #726969;}#direitosReservados{padding: 20px 0px 20px 0px; color: #FFFFFF; font-size: 15px; text-align: center;}</style></head><body> <center class='containerGeral'> <table cellpadding='0' cellspacing='0'> <tbody> <tr class='linha cabecalho'> <td class='logomarca'> <img id='logo' width='220' src='http://midiatest.teknisa.com/files/tkOrg058b72c1f04e58/Sup/MailImg/5b16e099e4679.png'> </td></tr><tr class='linha tipoEmail'> <td id='assunto'> <p>". $assunto ."</p></td></tr><div id='quadroConteudo'> <tr class='linha conteudo'> <td id='borda'> <center> <table cellpadding='0' cellspacing='0' style='background-color: #f2f2f2; width: 590px;'> <tbody> <tr> <td> <div style='margin: 20px; word-wrap: break-word;'> Os seguintes pedidos, enviados pelo fornecedor ". $fornecedor .", apresentaram erros durante a importação. Para verificar o relatório de erros, clique no número do pedido. </br></br> ". $mensagem ." </div></td></tr></tbody> </table> </td></tr></div><tr class='linha rodape'> <td id='direitosReservados'> <p>Todos os direitos reservados.</p></td></tr></tbody> </table></body></html>";
			
			$this->service->enviaEmail($assunto, $email, $destinatario);
			
		}
		
	}
	
	
}